// --------------------------------------------------------
// Objektorientierte Programmierung - Hausaufgabe 8
// Name:
// --------------------------------------------------------

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;

// --------------------------------------------------------
// Hier Ihre Klasse Artikel:
// --------------------------------------------------------


// --------------------------------------------------------
// Rahmenprogramm zum Einlesen der Daten:
// --------------------------------------------------------

class ArtikelTest {

	// Name der Daten-Datei:
	private static final String DATA_FILE = "Artikel.txt";

	// Maximale Anzahl verschiedener Artikel:
	private static final int MAX_ART = 100;

	// Array zum Speichern bisher erzeugter Artikel-Objekte:
	private static Artikel[] artArr;

	// Anzahl Artikel (d.h. erste freie Position im Array):
	private static int artNum;

	// Hilfsfunktion: Bestimmt Position des naechsten Trennzeichens |
	private static int nextDelimiter(int lineNo, String line, int from) {
		int to = line.indexOf('|', from);
		if(to < 0) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"| fehlt.");
			System.exit(3);
		}
		return to;
	}

	// Hilfsfunktion: Preis von String in Cent-Wert umrechnen:
	private static int parseMoney(int lineNo, String line, String money) {
		int point = money.indexOf('.');
		if(point < 0) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"'.' fehlt im Preis.");
			System.exit(3);
		}
		if(point != money.length() - 3) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"Preis muss genau zwei " +
					"Nachkommastellen haben.");
			System.exit(3);
		}
		String centStr = money.substring(point + 1);
		String euroStr = money.substring(0, point);
		int cent = 0;
		int euro = 0;
		try {
			cent = Integer.parseInt(centStr);
			euro = Integer.parseInt(euroStr);
		}
		catch(NumberFormatException e) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"Preis nicht numerisch.");
			System.exit(3);
		}
		if(cent < 0) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"Syntaxfehler im Preis.");
			System.exit(3);
		}
		if(euro < 0) {
			System.err.println("Fehler in Zeile " + lineNo +
					"(" + line + "): " +
					"Negativer Preis geht nicht.");
			System.exit(3);
		}

		// Ok:
		return euro * 100 + cent;
	}

	// Eingabezeile aus Daten-Datei verarbeiten:
	private static void processLine(int lineNo, String line) {
		// Erstes Feld im Datensatz: Hersteller/Importeur
		int from = 0;
		int to = nextDelimiter(lineNo, line, from);
		String manuf = line.substring(from, to);

		// Zweites Feld im Datensatz: ArtikelName
		from = to + 1;
		to = nextDelimiter(lineNo, line, from);
		String articlename = line.substring(from, to);

		// Drittes Feld im Datensatz: Preis
		from = to + 1;
		to = nextDelimiter(lineNo, line, from);
		String price = line.substring(from, to);

		// Viertes Feld im Datensatz: Anbieter
		from = to + 1;
		String shop = line.substring(from);

		// Preis pruefen und in Cent umrechnen:
		int cent = parseMoney(lineNo, line, price);

		// Testausgabe:
		//System.out.println(manuf + ":" + articlename + ":" +
		//			cent + ":" + shop);

		// Pruefen, ob Artikel neu:
		boolean found = false;
		for(int i = 0; i < artNum; i++) {
			Artikel art = artArr[i];
			if(art.artikelname().equals(articlename) &&
				art.hersteller().equals(manuf)) {
				art.alternativAngebot(cent, shop);
				found = true;
				break;
			}
		}

		// Ggf. Artikel-Objekt anlegen:
		if(!found) {
			if(artNum == MAX_ART) {
				System.err.println("Zu viele Artikel!");
				System.exit(4);
			}
			artArr[artNum++] =
				new Artikel(manuf, articlename, cent, shop);
		}
	}

	public static void main(String[] args) {

		// Array fuer Artikelliste anlegen:
		artArr = new Artikel[MAX_ART];
		artNum = 0;

		// Pruefungen der Eingabedatei
		// (hauptsaechlich, um File-Funktionen zu zeigen):
		File dataFile = new File(DATA_FILE);
		if(!dataFile.exists()) {
			System.err.println("Datei '" + DATA_FILE +
				"' existiert nicht.");
			System.exit(2);
		}
		if(!dataFile.isFile()) {
			System.err.println("'" + DATA_FILE +
				"' ist keine normale Datei " +
				"(z.B. Verzeichnis).");
			System.exit(2);
		}
		if(!dataFile.canRead()) {
			System.err.println("'" + DATA_FILE +
				"' ist nicht lesbar " +
				"(bitte Zugriffsrechte pruefen).");
			System.exit(2);
		}

		// Datei oeffnen:
		FileReader reader = null;
		try {
			reader = new FileReader(dataFile);
		}
		catch(FileNotFoundException e) {
			System.err.println("Fehler beim Eroeffnen " +
				"der Datei '" + DATA_FILE + "': " + e);
			System.exit(2);
		}
		BufferedReader input = new BufferedReader(reader);

		// Eingabezeilen lesen:
		int lineNo = 1;
		while(true) {
			String line = null;
			try {
				line = input.readLine();
			}
			catch(IOException e) {
				System.err.println("Fehler beim Lesen " +
					"aus der Datei '" + DATA_FILE +
					"': " + e);
				System.exit(2);
			}
			// Schluss falls Dateiende erreicht:
			if(line == null)
				break;
			processLine(lineNo, line);
			lineNo++;
		}

		// BufferedReader schliessen, schliesst auch FileReader:
		try {
			input.close();
		}
		catch(IOException e) {
			System.err.println("Fehler beim Schliessen " +
				"der Datei '" + DATA_FILE + "': " + e);
			System.exit(2);
		}

		// Artikelliste drucken:
		System.out.println("*** " + artNum + " Artikel ***");
		System.out.println();
		for(int i = 0; i < artNum; i++)
			System.out.println(artArr[i]);
	}
}
